//
//  MATCOperations.m
//  ThreadingExample
//
//  Created by Brad Larson on 3/6/2010.
//

#import "MATCOperations.h"
#import "MATCCalculationOperation.h"

@implementation MATCOperations

#pragma mark -
#pragma mark Initialization and teardown

- (void)viewDidLoad 
{
    [super viewDidLoad];

	calculationQueue = [[NSOperationQueue alloc] init];
//	[calculationQueue setMaxConcurrentOperationCount:1];
	
	
	subtitles = [[NSMutableArray alloc] initWithObjects:@"Not started", @"Not started", @"Not started", @"Not started", @"Not started", @"Not started", nil];

	UIBarButtonItem *startButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Start" style:UIBarButtonItemStylePlain target:self action:@selector(startCalculation)];
	self.navigationItem.rightBarButtonItem = startButtonItem;
	[startButtonItem release];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleOperationStart:) name:kMATCCalculationStartedNotification object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleOperationCompletion:) name:kMATCCalculationFinishedNotification object:nil];
}

- (void)dealloc 
{
	[calculationQueue release];
    [super dealloc];
}

#pragma mark -
#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
    return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section 
{
    return [subtitles count];;
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
	
	cell.textLabel.text = [NSString stringWithFormat:@"Operation %d", indexPath.row];
	
	NSString *currentOperationState = [subtitles objectAtIndex:indexPath.row];
	cell.detailTextLabel.text = currentOperationState;
	if ([currentOperationState isEqualToString:@"Completed"])
	{
		cell.accessoryType = UITableViewCellAccessoryCheckmark;
	}
	else
	{
		cell.accessoryType = UITableViewCellAccessoryNone;
	}
    
    return cell;
}

#pragma mark -
#pragma mark Calculation methods

- (IBAction)startCalculation;
{
	MATCCalculationOperation *calculationOperation1 = [[MATCCalculationOperation alloc] initWithCycles:10000 operationID:0];
	MATCCalculationOperation *calculationOperation2 = [[MATCCalculationOperation alloc] initWithCycles:10000 operationID:1];
	MATCCalculationOperation *calculationOperation3 = [[MATCCalculationOperation alloc] initWithCycles:10000 operationID:2];
	MATCCalculationOperation *calculationOperation4 = [[MATCCalculationOperation alloc] initWithCycles:10000 operationID:3];
	MATCCalculationOperation *calculationOperation5 = [[MATCCalculationOperation alloc] initWithCycles:10000 operationID:4];
	MATCCalculationOperation *calculationOperation6 = [[MATCCalculationOperation alloc] initWithCycles:10000 operationID:5];

//	[calculationOperation2 addDependency:calculationOperation1];
//	[calculationOperation3 addDependency:calculationOperation2];
//	[calculationOperation4 addDependency:calculationOperation3];
//	[calculationOperation5 addDependency:calculationOperation4];
//	[calculationOperation6 addDependency:calculationOperation5];
	
	[calculationOperation6 setQueuePriority:NSOperationQueuePriorityHigh];
	[calculationOperation5 setQueuePriority:NSOperationQueuePriorityVeryHigh];
	[calculationOperation2 setQueuePriority:NSOperationQueuePriorityLow];
	
	[calculationQueue addOperation:calculationOperation1];
	[calculationQueue addOperation:calculationOperation2];
	[calculationQueue addOperation:calculationOperation3];
	[calculationQueue addOperation:calculationOperation4];
	[calculationQueue addOperation:calculationOperation5];
	[calculationQueue addOperation:calculationOperation6];

	[calculationOperation1 release];
	[calculationOperation2 release];
	[calculationOperation3 release];
	[calculationOperation4 release];
	[calculationOperation5 release];
	[calculationOperation6 release];	
}

- (void)handleOperationStart:(NSNotification *)note;
{
	NSInteger indexOfOperation = [[note object] integerValue];
	[subtitles removeObjectAtIndex:indexOfOperation];
	[subtitles insertObject:@"Executing" atIndex:indexOfOperation];
	
	NSIndexPath *updatedIndexPath = [NSIndexPath indexPathForRow:indexOfOperation inSection:0];
	[self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:updatedIndexPath] withRowAnimation:UITableViewRowAnimationNone];
}

- (void)handleOperationCompletion:(NSNotification *)note;
{
	NSInteger indexOfOperation = [[note object] integerValue];
	[subtitles removeObjectAtIndex:indexOfOperation];
	[subtitles insertObject:@"Completed" atIndex:indexOfOperation];
	
	NSIndexPath *updatedIndexPath = [NSIndexPath indexPathForRow:indexOfOperation inSection:0];
	[self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:updatedIndexPath] withRowAnimation:UITableViewRowAnimationNone];
}

@end

